package cn.com.qmth.stmms.common.utils;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFFooter;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Comment;
import org.apache.poi.ss.usermodel.DataFormat;
import org.apache.poi.ss.usermodel.Font;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFClientAnchor;
import org.apache.poi.xssf.usermodel.XSSFRichTextString;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.common.collect.Lists;

import cn.com.qmth.stmms.common.annotation.ExcelField;

/**
 * 导出Excel文件（导出“XLSX”格式，支持大数据量导出 @see org.apache.poi.ss.SpreadsheetVersion）
 * 
 * @author XiongKaiJun
 * @version 2016-06-12
 */
public class ExportExcel2 {

    private static Logger log = LoggerFactory.getLogger(ExportExcel2.class);

    /**
     * 工作薄对象
     */
    private HSSFWorkbook wb;

    /**
     * 工作表对象
     */
    private Sheet sheet;

    /**
     * 样式列表
     */
    private Map<String, CellStyle> styles;

    /**
     * 当前行号
     */
    private int rownum;

    /**
     * 注解列表（Object[]{ ExcelField, Field/Method }）
     */
    List<Object[]> annotationList = Lists.newArrayList();

    /**
     * 构造函数
     * 
     * @param title
     *            表格标题，传“空值”，表示无标题
     * @param cls
     *            实体对象，通过annotation.ExportField获取标题
     */
    public ExportExcel2(String title, String subTitle, Class<?> cls) {
        this(title, subTitle, cls, 1);
    }

    /**
     * 构造函数
     * 
     * @param title
     *            表格标题，传“空值”，表示无标题
     * @param cls
     *            实体对象，通过annotation.ExportField获取标题
     * @param type
     *            导出类型（1:导出数据；2：导出模板）
     * @param groups
     *            导入分组
     */
    public ExportExcel2(String title, String subTitle, Class<?> cls, int type, int... groups) {
        // Get annotation field
        Field[] fs = cls.getDeclaredFields();
        for (Field f : fs) {
            ExcelField ef = f.getAnnotation(ExcelField.class);
            if (ef != null && (ef.type() == 0 || ef.type() == type)) {
                if (groups != null && groups.length > 0) {
                    boolean inGroup = false;
                    for (int g : groups) {
                        if (inGroup) {
                            break;
                        }
                        for (int efg : ef.groups()) {
                            if (g == efg) {
                                inGroup = true;
                                annotationList.add(new Object[] { ef, f });
                                break;
                            }
                        }
                    }
                } else {
                    annotationList.add(new Object[] { ef, f });
                }
            }
        }
        // Get annotation method
        Method[] ms = cls.getDeclaredMethods();
        for (Method m : ms) {
            ExcelField ef = m.getAnnotation(ExcelField.class);
            if (ef != null && (ef.type() == 0 || ef.type() == type)) {
                if (groups != null && groups.length > 0) {
                    boolean inGroup = false;
                    for (int g : groups) {
                        if (inGroup) {
                            break;
                        }
                        for (int efg : ef.groups()) {
                            if (g == efg) {
                                inGroup = true;
                                annotationList.add(new Object[] { ef, m });
                                break;
                            }
                        }
                    }
                } else {
                    annotationList.add(new Object[] { ef, m });
                }
            }
        }
        // Field sorting
        Collections.sort(annotationList, new Comparator<Object[]>() {

            public int compare(Object[] o1, Object[] o2) {
                return new Integer(((ExcelField) o1[0]).sort()).compareTo(new Integer(((ExcelField) o2[0]).sort()));
            };
        });
        // Initialize
        List<String> headerList = Lists.newArrayList();
        for (Object[] os : annotationList) {
            String t = ((ExcelField) os[0]).title();
            // 如果是导出，则去掉注释
            if (type == 1) {
                String[] ss = StringUtils.split(t, "**", 2);
                if (ss.length == 2) {
                    t = ss[0];
                }
            }
            headerList.add(t);
        }
        initialize(title, subTitle, headerList);
    }

    /**
     * 构造函数
     * 
     * @param title
     *            表格标题，传“空值”，表示无标题
     * @param headers
     *            表头数组
     */
    public ExportExcel2(String title, String subTile, String[] headers) {
        initialize(title, subTile, Lists.newArrayList(headers));
    }

    /**
     * 构造函数
     * 
     * @param title
     *            表格标题，传“空值”，表示无标题
     * @param headerList
     *            表头列表
     */
    public ExportExcel2(String title, String subTile, List<String> headerList) {
        initialize(title, subTile, headerList);
    }

    /**
     * 初始化函数
     * 
     * @param title
     *            表格标题，传“空值”，表示无标题
     * @param headerList
     *            表头列表
     */
    @SuppressWarnings("static-access")
    private void initialize(String title, String subTile, List<String> headerList) {
        this.wb = new HSSFWorkbook();
        this.sheet = wb.createSheet("Export");
        sheet.setMargin(sheet.BottomMargin, ScoreExportParam.SHEET_BOTTOMMARGIN);
        sheet.setMargin(sheet.LeftMargin, ScoreExportParam.SHEET_LEFTMARGIN);
        sheet.setMargin(sheet.RightMargin, ScoreExportParam.SHEET_RIGHTMARGIN);
        sheet.setMargin(sheet.TopMargin, ScoreExportParam.SHEET_TOPMARGIN);
        sheet.setMargin(sheet.HeaderMargin, ScoreExportParam.SHEET_HEADERMARGIN);
        sheet.setMargin(sheet.FooterMargin, ScoreExportParam.SHEET_FOOTERMARGIN);
        HSSFFooter footer = (HSSFFooter) sheet.getFooter();
        footer.setCenter("第" + footer.page() + "页，共" + footer.numPages() + "页");// 设置页脚
        this.styles = createStyles(wb);
        // Create title
        if (StringUtils.isNotBlank(title)) {
            Row titleRow = sheet.createRow(rownum++);
            // 设置标题行高
            titleRow.setHeightInPoints((float) 22.6);
            Cell titleCell = titleRow.createCell(0);
            titleCell.setCellStyle(styles.get("title"));
            titleCell.setCellValue(title);
            sheet.addMergedRegion(new CellRangeAddress(titleRow.getRowNum(), titleRow.getRowNum(), titleRow.getRowNum(),
                    headerList.size() - 1));
        }
        if (headerList == null) {
            throw new RuntimeException("headerList not null!");
        }

        Row row1 = sheet.createRow(rownum++);
        row1.setHeightInPoints((float) 22.6);
        Cell cell2 = row1.createCell(0);
        cell2.setCellStyle(styles.get("subTile"));
        cell2.setCellValue(subTile);
        sheet.addMergedRegion(
                new CellRangeAddress(row1.getRowNum(), row1.getRowNum(), row1.getRowNum() - 1, headerList.size() - 1));

        Row headerRow = sheet.createRow(rownum++);
        headerRow.setHeightInPoints((float) 22.6);
        for (int i = 0; i < headerList.size(); i++) {
            Cell cell = headerRow.createCell(i);
            cell.setCellStyle(styles.get("header"));
            String[] ss = StringUtils.split(headerList.get(i), "**", 2);
            if (ss.length == 2) {
                cell.setCellValue(ss[0]);
                Comment comment = this.sheet.createDrawingPatriarch()
                        .createCellComment(new XSSFClientAnchor(0, 0, 0, 0, (short) 3, 3, (short) 5, 6));
                comment.setString(new XSSFRichTextString(ss[1]));
                cell.setCellComment(comment);
            } else {
                cell.setCellValue(headerList.get(i));
            }
            sheet.autoSizeColumn(i);
        }
        for (int i = 0; i < headerList.size(); i++) {
            String value = headerList.get(i);
            if (ScoreExportParam.SPECIAL_KSMC.equals(value)) {
                sheet.setColumnWidth(i, 10480); // 8cm
            }
            if (ScoreExportParam.SPECIAL_KCMC.equals(value) || ScoreExportParam.SPECIAL_YXXH.equals(value)) {
                sheet.setColumnWidth(i, 3930); // 3cm
            }
            if (ScoreExportParam.SPECIAL_XSXM.equals(value) || ScoreExportParam.SPECIAL_KSCJ.equals(value)) {
                sheet.setColumnWidth(i, 2090); // 1.6cm
            }
            if (ScoreExportParam.SPECIAL_XH.equals(value)) {
                sheet.setColumnWidth(i, 1048); // 0.8cm
            }

        }
        log.debug("Initialize success.");
    }

    /**
     * 创建表格样式
     * 
     * @param wb
     *            工作薄对象
     * @return 样式列表
     */
    private Map<String, CellStyle> createStyles(Workbook wb) {
        Map<String, CellStyle> styles = new HashMap<String, CellStyle>();

        CellStyle style = wb.createCellStyle();
        style.setAlignment(CellStyle.ALIGN_CENTER);
        style.setVerticalAlignment(CellStyle.VERTICAL_CENTER);
        Font titleFont = wb.createFont();
        titleFont.setFontName("宋体");
        titleFont.setFontHeightInPoints((short) 16);
        titleFont.setBoldweight(Font.BOLDWEIGHT_BOLD);
        style.setFont(titleFont);
        styles.put("title", style);

        style = wb.createCellStyle();
        style.setVerticalAlignment(CellStyle.VERTICAL_CENTER);
        style.setBorderRight(CellStyle.BORDER_THIN);
        style.setRightBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
        style.setBorderLeft(CellStyle.BORDER_THIN);
        style.setLeftBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
        style.setBorderTop(CellStyle.BORDER_THIN);
        style.setTopBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
        style.setBorderBottom(CellStyle.BORDER_THIN);
        style.setBottomBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
        Font dataFont = wb.createFont();
        dataFont.setFontName("宋体");
        dataFont.setFontHeightInPoints((short) 10);
        style.setFont(dataFont);
        styles.put("data", style);

        style = wb.createCellStyle();
        style.cloneStyleFrom(styles.get("data"));
        style.setAlignment(CellStyle.ALIGN_LEFT);
        styles.put("data1", style);

        style = wb.createCellStyle();
        style.cloneStyleFrom(styles.get("data"));
        style.setAlignment(CellStyle.ALIGN_CENTER);
        styles.put("data2", style);

        style = wb.createCellStyle();
        style.cloneStyleFrom(styles.get("data"));
        style.setAlignment(CellStyle.ALIGN_RIGHT);
        styles.put("data3", style);

        style = wb.createCellStyle();
        style.cloneStyleFrom(styles.get("data"));
        style.setAlignment(CellStyle.ALIGN_CENTER);
        Font headerFont = wb.createFont();
        headerFont.setFontName("宋体");
        headerFont.setFontHeightInPoints((short) 10);
        headerFont.setBoldweight(Font.BOLDWEIGHT_BOLD);
        style.setFont(headerFont);
        styles.put("header", style);

        style = wb.createCellStyle();
        style.cloneStyleFrom(styles.get("title"));
        Font subFont = wb.createFont();
        subFont.setFontName("宋体");
        subFont.setFontHeightInPoints((short) 14);
        subFont.setBoldweight(Font.BOLDWEIGHT_BOLD);
        style.setFont(subFont);
        styles.put("subTile", style);

        return styles;
    }

    /**
     * 添加一行
     * 
     * @return 行对象
     */
    public Row addRow() {
        return sheet.createRow(rownum++);
    }

    /**
     * 添加一个单元格
     * 
     * @param row
     *            添加的行
     * @param column
     *            添加列号
     * @param val
     *            添加值
     * @return 单元格对象
     */
    public Cell addCell(Row row, int column, Object val) {
        return this.addCell(row, column, val, 0, Class.class);
    }

    /**
     * 添加一个单元格
     * 
     * @param row
     *            添加的行
     * @param column
     *            添加列号
     * @param val
     *            添加值
     * @param align
     *            对齐方式（1：靠左；2：居中；3：靠右）
     * @return 单元格对象
     */
    public Cell addCell(Row row, int column, Object val, int align, Class<?> fieldType) {
        Cell cell = row.createCell(column);
        CellStyle style = styles.get("data" + (align >= 1 && align <= 3 ? align : ""));
        try {
            if (val == null) {
                cell.setCellValue("");
            } else if (val instanceof String) {
                cell.setCellValue((String) val);
            } else if (val instanceof Integer) {
                cell.setCellValue((Integer) val);
            } else if (val instanceof Long) {
                cell.setCellValue((Long) val);
            } else if (val instanceof Double) {
                cell.setCellValue((Double) val);
            } else if (val instanceof Float) {
                cell.setCellValue((Float) val);
            } else if (val instanceof Date) {
                DataFormat format = wb.createDataFormat();
                style.setDataFormat(format.getFormat("yyyy-MM-dd"));
                cell.setCellValue((Date) val);
            } else {
                if (fieldType != Class.class) {
                    cell.setCellValue((String) fieldType.getMethod("setValue", Object.class).invoke(null, val));
                } else {
                    cell.setCellValue((String) Class
                            .forName(this.getClass().getName().replaceAll(this.getClass().getSimpleName(),
                                    "fieldtype." + val.getClass().getSimpleName() + "Type"))
                            .getMethod("setValue", Object.class).invoke(null, val));
                }
            }
        } catch (Exception ex) {
            log.info("Set cell value [" + row.getRowNum() + "," + column + "] error: " + ex.toString());
            cell.setCellValue(val.toString());
        }
        cell.setCellStyle(style);
        return cell;
    }

    /**
     * 添加数据（通过annotation.ExportField添加数据）
     * 
     * @return list 数据列表
     */
    public <E> ExportExcel2 setDataList(List<E> list) {
        for (E e : list) {
            int colunm = 0;
            Row row = this.addRow();
            row.setHeightInPoints((float) 22.6);// 设置行高 0.8cm
            StringBuilder sb = new StringBuilder();
            for (Object[] os : annotationList) {
                ExcelField ef = (ExcelField) os[0];
                Object val = null;
                // Get entity value
                try {
                    if (StringUtils.isNotBlank(ef.value())) {
                        val = Reflections.invokeGetter(e, ef.value());
                    } else {
                        if (os[1] instanceof Field) {
                            val = Reflections.invokeGetter(e, ((Field) os[1]).getName());
                        } else if (os[1] instanceof Method) {
                            val = Reflections.invokeMethod(e, ((Method) os[1]).getName(), new Class[] {},
                                    new Object[] {});
                        }
                    }
                } catch (Exception ex) {
                    log.info(ex.toString());
                    val = "";
                }
                this.addCell(row, colunm++, val, ef.align(), ef.fieldType());
                sb.append(val + ", ");
            }
            log.debug("Write success: [" + row.getRowNum() + "] " + sb.toString());
        }
        return this;
    }

    /**
     * 输出数据流
     * 
     * @param os
     *            输出数据流
     */
    public ExportExcel2 write(OutputStream os) throws IOException {
        wb.write(os);
        os.close();
        return this;
    }

    /**
     * 输出到客户端
     * 
     * @param fileName
     *            输出文件名
     */
    public ExportExcel2 write(HttpServletResponse response, String fileName) throws IOException {
        response.reset();
        response.setContentType("application/octet-stream; charset=utf-8");
        response.setHeader("Content-Disposition", "attachment; filename=" + Encodes.urlEncode(fileName));
        write(response.getOutputStream());
        return this;
    }

    /**
     * 输出到文件
     * 
     * @param fileName
     *            输出文件名
     */
    public ExportExcel2 writeFile(String name) throws FileNotFoundException, IOException {
        FileOutputStream os = new FileOutputStream(name);
        this.write(os);
        return this;
    }

    /**
     * 导出测试
     */
    // public static void main(String[] args) throws Throwable {
    //
    // List<String> headerList = Lists.newArrayList();
    // headerList.add("序号");
    // headerList.add("考点名称");
    // headerList.add("课程名称");
    // headerList.add("院校学号");
    // headerList.add("学生姓名");
    // headerList.add("考试成绩");
    //
    // List<String> dataRowList = Lists.newArrayList();
    // for (int i = 1; i <= headerList.size()-1; i++) {
    // dataRowList.add("数据"+i);
    // }
    //
    // List<List<String>> dataList = Lists.newArrayList();
    // for (int i = 1; i <=20; i++) {
    // dataList.add(dataRowList);
    // }
    //
    // ExportExcel2 ee = new ExportExcel2("[考试批次名称]成绩单","负责教师签字：_______________
    // 交回日期：_______________", headerList);
    //
    // for (int i = 0; i < dataList.size(); i++) {
    // Row row = ee.addRow();
    // row.setHeightInPoints((float)22.6);
    // ee.addCell(row, 0, i);
    // for (int j = 0;j < dataList.get(i).size(); j++) {
    // ee.addCell(row, j+1, dataList.get(i).get(j));
    // }
    // }
    //
    // ee.writeFile("target/export.xls");
    //
    // log.debug("Export success.");
    //
    // }

}
